# Filename:        plotters.R           
# Date:            2017-03-08
# Author:          Kristian Sandaker (kristian.sandaker@outlook.com)
# Purpose:         Functions for e.g. plotting
# Data Written:    
# Packages Used:   gridExtra, lattice, latticeExtra, lubridate
# Source Used:     helpers.R

library(lattice)
library(lubridate)
source('helpers.R')

plotPriceFigures <- function(all=F, dataFrameAddress, outputNamesDataFrameAddress=outputNamesDataFrameAddress,
                             lagDataFrameAddress=lagDataFrameAddress, startDate = as.Date('2005-01-01'),
                             endDate = as.Date('2017-01-31'),
                             figureFeatures=c('ts', 'logrets', 'hist'),
                             ncol=3, plotName='time-series-comboplot-%02d',
                             oneFile=F, addLegend=F,
                             plotDir='/Users/Kristian/Kristian/Dropbox/Apps/ShareLaTeX/Masteroppgave/plots/appendix/') {
  df_timeSeries <- importDataFrame(dataFrameAddress=dataFrameAddress, 
                                   startDate=startDate, endDate=endDate,
                                   convertToReturns='')
  df_logReturns <- importDataFrame(dataFrameAddress=dataFrameAddress, 
                                   startDate=startDate, endDate=endDate,
                                   convertToReturns='log')
  # Select particular column names
  if (!all) {
    relevantSeries <- c("date")
    relevantSeriesNames <- c()
    for (i in 2:dim(df_logReturns)[2]) {
      seriesName <- colnames(df_logReturns)[i]
      df_lagStructure_index <- which(df_lagStructure[,1]==seriesName)
      if (sum(df_lagStructure[df_lagStructure_index,2:dim(df_lagStructure)[2]] > 0) > 0) {
        relevantSeries <- c(relevantSeries, df_lagStructure[df_lagStructure_index,1])
        relevantSeriesNames <- c(relevantSeriesNames, df_outputNames[df_lagStructure_index,2])
      }
    }
    df_timeSeries <- df_timeSeries[relevantSeries]
    df_logReturns <- df_logReturns[relevantSeries]
  }
  
  # Date correction
  df_timeSeries <- df_timeSeries[which(df_timeSeries[,1] >= startDate & df_timeSeries[,1] <= endDate),]
  df_logReturns <- df_logReturns[which(df_logReturns[,1] >= startDate & df_logReturns[,1] <= endDate),]
  
  dates_timeSeries <- df_timeSeries[,1]
  dates_logReturns <- df_logReturns[,1]
  
  # Graphics settings
  cexAxis <- 1.3
  cexLab <- 1.3
  cexMain <- 1.3
  cexLegend <- 1.0
  cexTitle <- 1.0
  parMar <- c(5,5,5,5)
  
  # Number of observations
  N <- dim(df_logReturns)[2]
  nrow <- (N - N %% 6)/6 + (N %% 6 != 0)
  
  # Plot limits
  xlim_timeSeries <- c(min(dates_timeSeries), max(dates_timeSeries))
  xlim_logReturns <- c(min(dates_logReturns), max(dates_logReturns))
  # Plot axes
  dateAxt_timeSeries <- seq(min(dates_timeSeries), max(dates_timeSeries), by="year")
  dateAxt_logReturns <- seq(min(dates_logReturns), max(dates_logReturns), by="year")
  
  pdf(paste(plotDir, plotName, '.pdf', sep=""), height=13,
      width=9, family = "Palatino", onefile=oneFile)
  par(mfrow=c(nrow, ncol), mar=parMar, cex.axis=cexAxis, cex.lab=cexLab,
      cex.main=cexMain)
  for (i in 2:N) {
    for (j in 1:length(figureFeatures)) {
      feature <- figureFeatures[j]
      if (feature == 'ts') {
        plot(df_timeSeries[,i] ~ dates_timeSeries, type='l', xlim=xlim_timeSeries, col='black',
             lwd=0.5, xlab='Time', ylab='',
             main='', xaxt='n', bty='n')
        axis(side=1, at=dateAxt_timeSeries, labels=format(dateAxt_timeSeries, '%Y'))
      } else if (feature == 'logrets') {
        plot(df_logReturns[,i] ~ dates_logReturns, type='l', xlim=xlim_timeSeries, col='black',
             lwd=0.5, xlab='Time', ylab='Log return',
             main='', xaxt='n', bty='n')
        axis(side=1, at=dateAxt_logReturns, labels=format(dateAxt_logReturns, '%Y'))
      } else if (feature == 'hist') {
        hist(df_logReturns[,i], main='', xlab='Log return',
             ylab='Rel. frequency [%]', freq=F, breaks=20,
             density=15, angle=45, col='grey', border='black')
        mu <- mean(df_logReturns[,i])
        sigma <- sd(df_logReturns[,i])
        if (addLegend) {
          legend('topright', cex=cexLegend, lty=c(0,2), lwd=c(0,0.5),
                 col=c('white', 'red'), bty='n',
                 legend=c(as.expression(bquote(atop(mu ~ "=" ~ .(format(mu, digits=3, scientific=T)),
                                                    sigma ~ "=" ~ .(format(mu, digits=3, scientific=T))))), as.expression(bquote("Fitted Gaussian"))))
        }
        curve(dnorm(x, mean = mu, sd = sigma), add=TRUE, col = "red", lty = 2,
              lwd = 1) # Gaussian overlay
      }
      if (j == 2) {
       title(main=bquote(bold(.(relevantSeriesNames[i-1]))), cex.main=cexTitle)
      }
    }
    # mtext(bquote(bold(.(relevantSeriesNames[i-1]))), outer = TRUE, cex = cexTitle, line=-15*i, side=3)
  }
  dev.off()
}


plotMeanHorizonForecast <- function(df_forecast, startDate=as.Date("2016-02-29"), priceDf, forecastHorizon=12) {
  dates_df_forecast <- df_forecast[,1]
  dateIndex <- which(dates_df_forecast == startDate)
  dates_plot <- rollback(startDate)
  for (i in 1:forecastHorizon) {
    dates_plot <- c(dates_plot, rollback(dates_plot[i]+days(40)))
  }
  
  startPrice <- priceDf[priceDf[,1] == rollback(startDate),2]
  
  forecast_true <- c(startPrice, exp(as.numeric(df_forecast[dateIndex,2:(forecastHorizon+1)]))*startPrice)
  forecast_pred <- c(startPrice, exp(as.numeric(df_forecast[dateIndex,(forecastHorizon+2):dim(df_forecast)[2]]))*startPrice)
  
  cexLegend <- 1.1
  
  ylim = c(0.9*min(c(forecast_true, forecast_pred), na.rm=T), 
           1.1*max(c(forecast_true, forecast_pred), na.rm=T))
  xlim <- c(min(dates_plot), max(dates_plot))
  dateAxt <- seq(xlim[1], xlim[2], by="month")
  
  plot(forecast_true ~ dates_plot[1:length(forecast_true)], 
       type="l", col="black", lwd=1.5,
       xlim=xlim, ylim=ylim, bty="n", xaxt='n', xlab="Time", ylab="USD/kg")
  par(new=T)
  plot(forecast_pred ~ dates_plot[1:length(forecast_pred)],
       type="l", col="red", lwd=1.5, lty=2,
       xlim=xlim, ylim=ylim, yaxt="n", xaxt="n", bty="n",
       xlab="", ylab="")
  axis(side=1, at=dateAxt, labels=format(dateAxt, '%m-%Y'))
  legend('topleft', col=c('black', 'red'), lty=c(1,2), lwd=c(1.5, 1.5),
         legend=c('True price path', 
                  paste('Forecasted price path at ', dates_plot[1], sep="")),
         cex=cexLegend, bty="n")
}

plotAllCrossCorrelation3d <- function(dataFrameAddress, outputNamesDataFrameAddress,
                                      startDate=as.Date('2007-08-31'), endDate=as.Date('2017-01-31'),
                                      maxLag=24, maxWindow=12, forecastHorizon=12) {
  df_fullLagged <- importFullLaggedDataFrame(dataFrameAddress=dataFrameAddress, lagDataFrameAddress='', useLagDataFrame=F, maxLag=maxLag,
                                             startDate=as.Date('2000-01-01'), endDate=as.Date('2017-01-31'), firstColDateCol=T, dateFormat='%d/%m/%Y', convertToReturns='log',
                                             forecastHorizon=forecastHorizon, useTimeWindowsDataFrame=F, timeWindowsDataFrameAddress='',
                                             allowedTimeWindows=seq(1,12))
  plotCrossCorrelation3d(df_fullLagged, outputNamesDataFrameAddress=outputNamesDataFrameAddress,
                         compCols=c(2, forecastHorizon+1), maxLag=maxLag, maxWindow=maxWindow, comboPlot=T)
  plotCrossCorrelation3d(df_fullLagged, outputNamesDataFrameAddress=outputNamesDataFrameAddress,
                         compCols=c(2, forecastHorizon+1), maxLag=maxLag, maxWindow=maxWindow, comboPlot=F)
}

plotCrossCorrelation3d <- function(df_fullLagged, outputNamesDataFrameAddress,
                                   compCols=c(2,13), maxLag=24, maxWindow=12, comboPlot=F) {
  cexScales <- 1.5
  cexLab <- 1.5
  cexMain <- 1.5
  cexColorKey <- 1.5
  mainInfoText <- ""#"Cross-correlation over lag and time window\n"
  mainVsNewLine <- "\n"
  zoom <- 0.85
  
  df_outputNames <- importGroupStructureDataFrame(outputNamesDataFrameAddress, constraints=F)
  
  if (comboPlot) {
    library(gridExtra)
    library(grid)
  }
  
  for (i in compCols[1]:compCols[2]) {
    y_series <- df_fullLagged[1:(dim(df_fullLagged)[1]-i+1),i]
    y_series_name <- colnames(df_fullLagged)[i]
    forecast_ahead <- as.numeric(strsplit(y_series_name, split="fc.")[[1]][2]) + 1
    # y_series_plotname <- paste(strsplit(y_series_name, ".fc")[[1]][1], " - ", forecast_ahead, "-step ahead", sep="")
    y_series_plotname <- paste(df_outputNames[which(df_outputNames[,1] == strsplit(y_series_name, ".fc")[[1]][1]),2], " - ", forecast_ahead, "-step ahead", sep="")
    
    print(paste("### ", y_series_name, " ###", sep="")) ## Print!
    
    n_series <- (dim(df_fullLagged)[2] - 1 - (compCols[2] - compCols[1]))/(maxLag*maxWindow)
    startCompIndex <- compCols[2] + 1
    
    if (comboPlot) {
      plts <- list()
      cexScales <- 0.7
      cexLab <- 0.7
      cexMain <- 0.9
      mainInfoText <- ""
      # mainVsNewLine <- "\n"
      cexColorKey <- 1.1
      zoom <- 1.0
    }
    for (j in 1:n_series) {
      x_series_name <- strsplit(colnames(df_fullLagged)[startCompIndex], split="_lag")[[1]][1]
      x_series_plotname <- df_outputNames[which(df_outputNames[,1] == x_series_name), 2]
      print(paste("Series computation: ", x_series_name, sep="")) ## Print!
      
      ccfs <- matrix(0, maxLag, maxWindow)
      
      for (k in 1:maxLag) {
        print(paste("  Lagging: ", k, sep="")) ## Print!
        for (l in 1:maxWindow) {
          seriesIndex <- startCompIndex + (maxWindow)*(k-1) + l - 1
          print(paste("   Considering: ", colnames(df_fullLagged)[seriesIndex], sep="")) ## Print!
          x_series <- df_fullLagged[1:(dim(df_fullLagged)[1]-i+1),seriesIndex]
          ccfs[k,l] <- cor(y_series, x_series)
        }
      }
      if (!comboPlot) {
        pdf(paste(getwd(), "/plots/3d/", y_series_name, "_3d_comp-series=", x_series_name, 
                  "_max-lag=", maxLag, "_max-window=", maxWindow, ".pdf", sep=""), 
            width=10, height=12, family='Palatino')
      }
      trellis.par.set("axis.line",list(col=NA,lty=1,lwd=1)) # Removes the frame
      plotZLab <- bquote("" ~ rho ~ "")
      plt <- wireframe(ccfs, row.values = seq(1:maxLag), col.values = seq(1:maxWindow),
                       scales = list(arrows = FALSE, col = "black", cex = cexScales), # z = list(arrows = TRUE, cex = cexScales, col = "black", tick.number = 5),
                       xlab = list("Lag", cex=cexLab), ylab = list("Time window", cex=cexLab), zlab = list("Corr.", cex=cexLab),
                       main = list(paste(mainInfoText, y_series_plotname, 
                                         " vs. ", mainVsNewLine, x_series_plotname, sep=""), cex = cexMain),
                       drape = TRUE, colorkey = list(space = 'bottom', height = .6, axis.line = list(col = 'black'), labels=list(cex = cexColorKey)),
                       screen = list(x = -50, y = -25, z = -10), zoom = zoom)
      if (comboPlot) {
        plts[[j]] <- plt
      }
      if (!comboPlot) {
        print(plt)
        dev.off()
      }
      
      startCompIndex <- startCompIndex + maxLag*maxWindow
    }
    if (comboPlot) {
      plot_num <- 1
      n_pages <- (length(plts) - length(plts) %% 15)/15 + 1
      for (pg in 1:n_pages) {
        pdf(paste(getwd(), "/plots/comboplots/", y_series_name, "_3d_comp-series", "_comboplot_page=", sprintf("%02d", pg), ".pdf", sep=""), 
            width=15, height=24, family='Palatino')
        grid.newpage()
        pushViewport(viewport(layout = grid.layout(5,3)))
        for (m in 1:5) {
          for (n in 1:3) {
            if (plot_num > length(plts)) {
              break()
            }
            pushViewport(viewport(layout.pos.row = m,
                                  layout.pos.col = n))
            print(plts[[plot_num]], newpage = FALSE)
            popViewport(1)
            plot_num <- plot_num + 1
          }
        }
        dev.off()
      }
    }

    # Update indices
    startCompIndex <- j + 1
  }
}

plotInSampleQrLines <- function(df_full, qrModels, Tau = c(0.05, 0.10, 0.20, 0.50, 0.80, 0.90, 0.95),
                                startDate=as.Date(df_full[2,1]), endDate=as.Date(df_full[(dim(df_full)[1]-12),1]),
                                outFileDir = paste(getwd(), "/plots/insample", sep=""),
                                addQuantileLines = F,
                                dataFrameAddress) {
  df_timeSeries <- importDataFrame(dataFrameAddress=dataFrameAddress, 
                                   startDate=as.Date('2005-01-01'),
                                   endDate=as.Date('2017-01-31'),
                                   convertToReturns='')
  df_timeSeries <- df_timeSeries[min(which(df_timeSeries[,1] %in% df_full[,1])):dim(df_timeSeries)[1],]
  rownames(df_timeSeries) <- seq(1:dim(df_timeSeries)[1])
  
  for (i in 1:dim(df_full)[1]) {
    date <- df_full[i,1]
    if ((date >= startDate) && (date <= endDate)) {
      # Construct the QR lines dataset
      qrLines <- data.frame(matrix(0, (1 + length(qrModels)), length(qrModels[[1]])))
      rownames(qrLines) <- df_full[(i-1):(i+length(qrModels)-1),1]
      colnames(qrLines) <- paste("Quantile ", Tau, sep="")
      for (j in 1:length(qrModels)) {
        for (k in 1:length(qrModels[[j]])) {
          if (j==1) {
            qrLines[1,k] <- df_timeSeries[(i-1),2]
          }
          qrModel <- qrModels[[j]][[k]]
          quantile_pred <- df_timeSeries[(i-1),2]*(1 + (exp(predict.rq(qrModel, df_full[i,])) - 1))
          qrLines[(j+1),k] <- quantile_pred
        }
      }
      ### Then plot the stuff
      ts_realized <- df_timeSeries[min(which(df_timeSeries[,1] %in% df_full[,1])):dim(df_timeSeries)[1],1:2]
      
      # Create the pdf
      pdf(paste(outFileDir, '/in-sample-qr-reg-horizons-', as.Date(rownames(qrLines)[1]), '.pdf', sep=''),
          height=8, width=14, family='Palatino')
      
      # pdf etc
      # Limits
      xlim <- c(min(as.Date(ts_realized[,1])), max(as.Date(ts_realized[,1]))+365)
      ylim <- c(round(min(min(qrLines), min(ts_realized[,2]))*0.9, digits=1),
                round(max(max(qrLines), max(ts_realized[,2]))*1.1, digits=1))
      
      # Top 7/8 of chart plot, bottom 1/8 of chart leged
      layout(rbind(1,2), heights=c(7,1))
      par(mar=c(5,5,5,5))
      
      # First define the plot area
      plot(ts_realized[,2] ~ ts_realized[,1], type="l", lty=1, col="white",
           xlab="Time", ylab="USD/kg", bty="n",
           main=paste("In-Sample Quantile Regression Horizons, Starting from ", 
                      format(as.Date(rownames(qrLines)[1]), "%b %d %Y"), sep=""),
           xlim=xlim, ylim=ylim, xaxt="n", yaxt="n")
      # Define the QR colors
      qrLinesColors <- c()
      availBlues <- seq(90, 40, by=-50/floor(dim(qrLines)[2]/2))
      for (j in 1:dim(qrLines)[2]) {
        if (j > (dim(qrLines)[2]/2)) {
          colorIndex <- dim(qrLines)[2]-j+1
        } else {
          colorIndex <- j
        }
        qrLinesColors <- c(qrLinesColors, hcl(240, 35, availBlues[colorIndex]))
        # print(paste("Plotting ", hcl(240, 35, availBlues[colorIndex]), sep=""))
      }
      # Then plot the QR lines
      for (j in 1:dim(qrLines)[2]) {
        linesDates <- as.Date(rownames(qrLines))
        # lines(linesDates, qrLines[,j], xlim=xlim, ylim=ylim, col=qrLinesColors[j])
        if (j > 1) {
          if (j > ceiling(dim(qrLines)[2]/2)) {
            polygonColor <- qrLinesColors[length(qrLinesColors)-j+2]
          } else {
            polygonColor <- qrLinesColors[j]
          }
          # print(paste("Drawing polygon ", j, ", color ", polygonColor, sep=""))
          polygon(c(linesDates, rev(linesDates)), c(qrLines[,j], rev(qrLines[,(j-1)])),
                  col=polygonColor, border="NA")
        }
      }
      # Add the dotted quantile line on top
      if (addQuantileLines) {
        linesDates <- as.Date(rownames(qrLines))
        for (j in 1:dim(qrLines)[2]) {
          lines(linesDates, qrLines[,j], xlim=xlim, ylim=ylim, col="white",
                lwd=1, lty=5)
        }
      }
      
      
      # Then plot the realized price on top
      par(new=T)
      plot(ts_realized[,2] ~ ts_realized[,1], type="l", lty=1, col="black",
            xlab="", ylab="", bty="n", main="", xlim=xlim, ylim=ylim,
            xaxt="n", yaxt="n", lwd=1.5)
      # Add axes
      dateAxt <- seq(xlim[1], xlim[2], by="year")
      axis(side=1, at=dateAxt, labels=format(dateAxt, '%b %Y'))
      priceAxt <- seq(ylim[1], ylim[2], 0.5)
      axis(side=2, at=priceAxt, labels=priceAxt)
      
      # Add vertical line to indicate forecast start
      ablineDate <- as.Date(rownames(qrLines)[1])
      abline(v=ablineDate, lty=5, lwd=1.5, col="red")# col=qrLinesColors[ceiling(length(qrLinesColors)/2)])
      text(x=5, y = 5, "Info. set")
      
      # Then add legend at bottom
      par(mar=c(0,0,0,0))
      quantileLegendTexts <- c()
      quantileLegendColors <- c()
      for (j in 2:length(Tau)) {
        quantileLegendTexts <- c(quantileLegendTexts, paste(Tau[j-1]*100, "-", Tau[j]*100, '% quantiles', sep=''))
        if (j > ceiling(length(Tau)/2)) {
          quantileLegendColors <- c(quantileLegendColors, qrLinesColors[length(Tau)-j+2])
        } else {
          quantileLegendColors <- c(quantileLegendColors, qrLinesColors[j])
        }
      }
      plot.new()
      legend('center', 'groups', c('NQSALMON USD/kg', quantileLegendTexts),
             fill=c('Black', quantileLegendColors), ncol=3, bty='n')
      
      # Then save pdf
      dev.off()
    }
  }
}

### 2017-04-21 START PLOT OF QR EXCEEDANCES ###
plotInSampleQrExceedances <- function(df_full, qrModels, Tau = c(0.10, 0.25, 0.50, 0.75, 0.90),
                                      startDate=as.Date(df_full[2,1]), endDate=as.Date(df_full[(dim(df_full)[1]-12),1]),
                                      latexDir = "/Users/Kristian/Kristian/Dropbox/Apps/ShareLaTeX/Masteroppgave/plots/in-sample-exceedances",
                                      startDate_timeSeries = as.Date('2005-01-01'), endDate_timeSeries = as.Date('2017-01-31'),
                                      outFileDir = paste(getwd(), "/plots/insample-exceedances", sep=""),
                                      dataFrameAddress) {
  df_timeSeries <- importDataFrame(dataFrameAddress=dataFrameAddress, startDate=startDate_timeSeries,
                                   endDate=endDate_timeSeries, convertToReturns='')[,1:2]
  
  require(lubridate)
  
  ## PLOT PARAMETRIZATION
  cexLab <- 1.8
  cexAxis <- 1.8
  cexLegend <- 1.8
  cexText <- 1.8
  cexTitle <- 2.3
  
  col1 <- "black"
  col2 <- "#800000"
  col3 <- "#000080"
  
  startDate_dfFull <- as.Date(df_full[1,1])
  endDate_dfFull <- df_full[max(which(!is.na(df_full[,1+length(qrModels)]))),1]
  # endDate_dfFull <- ymd(endDate_dfFull)
  # endDate_dfFull <- endDate_dfFull %m-% months(1)
  # day(endDate_dfFull) <- days_in_month(endDate_dfFull)
  
  endDate_laggedTimeSeries <- endDate_dfFull
  endDate_laggedTimeSeries <- ymd(endDate_laggedTimeSeries)
  endDate_laggedTimeSeries <- endDate_laggedTimeSeries %m-% months(1)
  day(endDate_laggedTimeSeries) <- days_in_month(endDate_laggedTimeSeries)
  
  startDate_qrModel <- as.Date(df_full[1,1])
  startDate_qrModel <- ymd(startDate_qrModel)
  startDate_qrModel <- startDate_qrModel %m-% months(1)
  day(startDate_qrModel) <- days_in_month(startDate_qrModel)
  
  endDate_qrModel <- df_full[max(which(!is.na(df_full[,1+length(qrModels)]))),1]
  
  for (i in 1:length(qrModels)) {
    for (j in 1:length(qrModels[[i]])) {
      
      pdf(paste(latexDir, '/in-sample-exceedance-plot-horizon=', i,
            '-tau=', format(round(Tau[j], 2), nsmall=2),'.pdf', sep=''),
          height=9, width=16, family='Palatino')
      
      qrModel <- qrModels[[i]][[j]]
      
      # Retrieve lagged log returns from the QR regression matrix
      qrModel_laggedLogReturns <- data.frame(date=df_timeSeries[which(df_timeSeries[,1] >= startDate_qrModel & df_timeSeries[,1] <= endDate_laggedTimeSeries),1],
                                              laglogreturn=df_full[which(df_full[,1] >= startDate_dfFull & df_full[,1] <= endDate_dfFull), 1+i])
      qrModel_laggedLogReturns[,1] <- as.Date(qrModel_laggedLogReturns[,1])
      
      # Retrieve the lagged time series
      qrModel_timeSeries <- df_timeSeries
      qrModel_laggedTimeSeries <- data.frame(date=qrModel_laggedLogReturns[,1],
                                            salmonprice_usdkg_nasdaq=df_timeSeries[which(df_timeSeries[,1] >= startDate_qrModel & 
                                                                                           df_timeSeries[,1] <= endDate_laggedTimeSeries)+i,2],
                                            stringsAsFactors=F)
      # data.frame(qrModel_laggedLogReturns, qrModel$fitted.values, qrModel$fitted.values + qrModel$residuals)
      
      qrModel_exceedances <- data.frame(date=qrModel_laggedTimeSeries[,1],
                                        exceedances=(qrModel$fitted.values > qrModel$fitted.values + qrModel$residuals))
      
      ## MAKE THE PLOTS
      layout(matrix(1:3, ncol = 1), widths = 1, heights = c(1.5,1.5,2), respect = FALSE)
      par(mar = c(0, 4.1, 4.1, 2.1)) # Bottom, left, top, right
      xlim_dates <- c(min(df_timeSeries[,1]), max(df_timeSeries[,1]))
      ylim_prices <- c(min(df_timeSeries[,2]), max(df_timeSeries[,2]))
      
      ## Initial NQSALMON time series
      plot(df_timeSeries[,2] ~ df_timeSeries[,1], type="l", col=col3, 
           xlim=xlim_dates, ylim=ylim_prices, ylab=NA, bty="n", xaxt="n", cex.lab=cexLab, cex.axis=cexAxis) # Tau
      title(bquote(atop(paste(bold("In-Sample Quantile Regression Exceedances, "), k, "=", .(i), ", ", tau, "=", .(paste("", format(round(Tau[j], 2), nsmall=2), sep="")), sep=""),
                        paste("Period: ", .(paste("", qrModel_laggedTimeSeries[1,1], sep="")), " - ", 
                              .(paste("", qrModel_laggedTimeSeries[dim(qrModel_laggedTimeSeries)[1],1], sep="")), sep=""))),
            cex.main=cexTitle)
      
      # Overlay lagged series
      par(new=T)
      plot(qrModel_laggedTimeSeries[,2] ~ qrModel_laggedTimeSeries[,1], type="l",
           col=col2, xaxt="n", xlim=xlim_dates, ylim=ylim_prices, main="", xlab=NA, ylab=NA, axes=F, bty="n")
      
      # Add legend
      legend("topleft", col=c(col3, col2), lty=c(1, 1), lwd=c(1,1), 
             legend=c("NQSALMON USD/kg", paste(i, "-step ahead value of NQSALMON USD/kg", sep="")),
             bg="white", cex=cexLegend)
      
      # Add date axis
      # 
      # axis(side=1, at=dateAxt, labels=format(dateAxt, '%b %Y'))
      
      ## Percentage and log return clustered bar chart
      par(mar = c(0, 4.1, 2.0, 2.1), bg=NA) # Bottom, left, top, right
      qrModel_relevantTimeSeries <- qrModel_timeSeries[which(qrModel_timeSeries[,1] %in% qrModel_laggedTimeSeries[,1]),]
      qrModel_returns <- data.frame(date=qrModel_laggedTimeSeries[,1],
                                         arithmeticReturn=((qrModel_laggedTimeSeries[,2]/qrModel_relevantTimeSeries[,2]) - 1),
                                         logReturn=(log(qrModel_laggedTimeSeries[,2]) - log(qrModel_relevantTimeSeries[,2])),
                                         qrModelQuantile=qrModel$fitted.values,
                                         qrModelLogReturn=qrModel_laggedLogReturns[,2],
                                         qrModelLogReturn2=qrModel$fitted.values+qrModel$residuals)
      ylim_logReturns <- c(min(qrModel_returns[,2:4]), max(qrModel_returns[,2:4]))
      
      # First plot the realized log returns
      plot(qrModel_returns[,3] ~ qrModel_returns[,1], type="l", col=col1,
           xlim=xlim_dates, ylim=ylim_logReturns, main="", xlab=NA, ylab=NA, xaxt="n", bty="n", lwd=1, lend="square", cex.lab=cexLab, cex.axis=cexAxis)      
      dateAxt <- seq(xlim_dates[1], xlim_dates[2], by = "6 months")
      axis(side=3, at=dateAxt, labels=format(dateAxt, '%b %Y'), cex.axis=cexAxis)
      
      # Then plot the QR model's log returns
      par(new=T)
      plot(qrModel_returns[,4] ~ qrModel_returns[,1], type="l", col=col2,
           xlim=xlim_dates, ylim=ylim_logReturns, main="", xlab=NA, ylab=NA, 
           xaxt="n", yaxt="n", bty="n", lwd=1)
      abline(a=0, b=0, lty=2, lwd=2, col="black")
      
      # Exceedances
      for (k in 1:dim(qrModel_exceedances)[1]) {
        if(qrModel_exceedances[k,2]) {
          abline(v=qrModel_exceedances[k,1], col="grey", lwd=2)
        }
      }
      
      legend("topleft", col=c(col1, col2), lty=c(1,1), lwd=c(1,1), legend=c(paste(i, "-step ahead realised NQSALMON USD/kg log return", sep=""),
                                                                                as.expression(bquote(paste(.(i), "-step ahead predicted NQSALMON USD/kg log return at quantile ", tau, "=" , .(Tau[j]), sep="")))),
             bg="white", cex=cexLegend)
      
      # Bottom right textbox with test scores
      source('latex_output.R') # For qrChristoffersen function
      
      # Christoffersen calculations (b/c lazy)
      fit <- qrModel$fitted.values > qrModel$fitted.values + qrModel$residuals
      n   <- length(fit)
      n_00 <- sum(as.logical(fit[1:(n-1)] == F) * as.logical(fit[2:n] == F))
      n_01 <- sum(as.logical(fit[1:(n-1)] == F) * as.logical(fit[2:n]))
      n_10 <- sum(as.logical(fit[1:(n-1)]) * as.logical(fit[2:n] == F))
      n_11 <- sum(as.logical(fit[1:(n-1)]) * as.logical(fit[2:n]))
      pi_obs <- sum(fit)/n
      pi_exp <- tau
      pi_01  <- n_01/(n_00+n_01)
      pi_11  <- n_11/(n_10+n_11)

      legend('bottomright', legend=do.call('expression', list(bquote(n == .(n)),
                                                              bquote(pi['01'] == .(format(round(pi_01, 2), nsmall=2))),
                                                              bquote(pi[11] == .(format(round(pi_11, 2), nsmall=2))),
                                                              bquote('Chr. test: ' ~ .(format(round(qrChristoffersen(qrModel), 2), nsmall=2))))),
           bg='white', cex=cexLegend)
      
      # Now plot the error
      par(mar = c(3.0, 4.1, 0.0, 2.1)) # Bottom, left, top, right
      error <- qrModel_returns[,3] - qrModel_returns[,4]
      # Blank plot
      plot(error ~ qrModel_returns[,1], type="n", col="black",
           xlim=xlim_dates, ylim=ylim_logReturns, main="", xlab=NA, ylab=NA, axes=F, bty="n", lwd=5, lend="square", cex.axis=cexAxis)
      
      
      abline(a=0, b=0, lty=1, lwd=1, col="black")
      axis(side=1, at=dateAxt, labels=format(dateAxt, '%b %Y'), cex.axis=cexAxis)
      
      # Exceedances (again)
      for (k in 1:dim(qrModel_exceedances)[1]) {
        if(qrModel_exceedances[k,2]) {
          abline(v=qrModel_exceedances[k,1], col="grey", lwd=2)
        }
      }
      par(new=T)
      plot(error ~ qrModel_returns[,1], type="h", col=col1,
           xlim=xlim_dates, ylim=ylim_logReturns, main="", xlab=NA, ylab=NA, xaxt="n", bty="n", lwd=5, lend="square", cex.axis=cexAxis)
      lines(error ~ qrModel_returns[,1], type="h", lwd=3, col="white")
      
      legend("topleft", col=col1, fill="white", legend=bquote(paste("Difference from ", tau, "=", .(Tau[j]), " quantile")),
                                                                                cex=cexLegend)
      
      dev.off() 
    }
  }
}



### 2017-05-09 START PLOT OF QR COEFFICIENTS SENSITIVITIES ###
plotQrCoefficientSensitivities <- function(df_full, gaFileName, Tau = c(0.10, 0.25, 0.50, 0.75, 0.90), df_outputNames, use_df_outputNames=TRUE,
                                             startDate=as.Date(df_full[2,1]), endDate=as.Date(df_full[(dim(df_full)[1]-12),1]), maxHorizons=12,
                                             latexDir = '/Users/Kristian/Kristian/Dropbox/Apps/ShareLaTeX/Masteroppgave/plots/in-sample-qr-sensitivities',
                                             outFileDir = paste(getwd(), "/plots/insample-qr-sensitivities", sep="")) {
  
  cexLab <- 1.5
  
  col1 <- "black"
  col2 <- "#800000"
  col3 <- "#000080"
  
  # Borrowed code from quantile_regression_model.constructQrModelsFromGaFile
  res_df <- read.table(file=gaFileName, header=TRUE, sep=",", 
                       stringsAsFactors = F, na.strings=c("", "NA"))
  horizon_series <- list()
  for (i in 1:maxHorizons) {
    k <- res_df[i,1]
    for (j in 2:dim(res_df)[2]) {
      if (j==2) {
        series_names <- res_df[i,j]
      } else if (!is.na(res_df[i,j])) {
        series_names <- c(series_names, res_df[i,j])
      }
    }
    phi_cols <- df_full[series_names]
    horizon_series[[k]] <- phi_cols
  }
  
  # Borrowed code form quantile_regression_model.constructQrModelsFromGaRun
  require(quantreg)
  rqsModels <- list()
  for (k in 1:length(horizon_series)) {
    phi_cols <- horizon_series[[k]]
    rqsModels[[k]] <- rq(data.frame(df_full[,1+k], phi_cols), tau=Tau) # Returns rqs - a process!
  }
  
  # Now for loop and plot the all horizons x parameters
  for (i in 1:length(rqsModels)) {
    for (j in 1:dim(rqsModels[[i]]$coefficients)[1]) {
      # First determine the coefficient name
      if (use_df_outputNames) {
        if (j == 1) {
          coeffName <- "Intercept"
        } else {
          print(paste("i=", i,"; Checking: ", rownames(rqsModels[[i]]$coefficients)[j], sep=""))
          seriesNames <- strsplit(rownames(rqsModels[[i]]$coefficients)[j], split="_lag.")[[1]]
          lag <- as.numeric(strsplit(seriesNames[2], split="_delta.")[[1]][1])
          delta <- if(strsplit(seriesNames[2], split="_delta.")[[1]][2]=="01") "MoM" else "YoY"
          coeffName <- paste(i, "-step ahead model\n", df_outputNames[which(df_outputNames[,1] == seriesNames[[1]][1]),2], " (lag ", lag, ", ", delta, ")", sep="")
        }
      } else {
        coeffName <- rownames(rqsModels[[i]]$coefficients)[j]
      }
      plotName <- gsub("_", "-", strsplit(rownames(rqsModels[[i]]$coefficients)[j], split="_lag.")[[1]][1])
      
      # Then plot the show
      pdf(paste(latexDir, '/qr-sensitivity-horizon=', sprintf("%02d", i), "-", plotName, '.pdf', sep=""),
          height=5, width=8, family='Palatino')
      modified.plot.summary.rqs(summary(rqsModels[[i]], se="boot", bsmethod="pwy", R=1000), parm=j,
                       ols=FALSE, xlab="Quantile", mar=c(5,5,5,5), cex.lab=cexLab, main=coeffName,
                       bty="n", xaxt="n", yaxt="n", cex=1, lineColor=col2, zeroLineColor=col3, olsColor=col2)
      dev.off()
    }
  }
}

modified.plot.summary.rqs <- function (x, parm = NULL, level = 0.9, ols = TRUE, mfrow = NULL, 
          mar = NULL, ylim = NULL, main = NULL, col = gray(c(0, 0.75)), 
          border = NULL, lcol = 2, lty = 1:2, cex = 0.5, pch = 20, 
          type = "b", xlab = "", ylab = "", xaxt="n", yaxt="n", bty="n", lineColor="red", olsColor=col3,
          zeroLineColor, ...) 
{
  zalpha <- qnorm(1 - (1 - level)/2)
  taus <- sapply(x, function(x) x$tau)
  cf <- lapply(x, coef)
  if (ncol(cf[[1]]) == 4) {
    for (i in 1:length(cf)) {
      cfi <- cf[[i]]
      cfi <- cbind(cfi[, 1], cfi[, 1] - cfi[, 2] * zalpha, 
                   cfi[, 1] + cfi[, 2] * zalpha)
      colnames(cfi) <- c("coefficients", "lower bd", "upper bd")
      cf[[i]] <- cfi
    }
  }
  if (ncol(cf[[1]]) != 3) 
    stop("summary.rqs components have wrong dimension")
  if (is.null(parm)) 
    parm <- rownames(cf[[1]])
  if (is.numeric(parm)) 
    parm <- rownames(cf[[1]])[parm]
  cf <- lapply(cf, function(x) x[parm, , drop = FALSE])
  names(cf) <- paste("tau=", taus)
  if (ols) {
    obj <- x[[1]]
    # mt <- terms(obj)
    # mf <- model.frame(obj)
    # y <- model.response(mf)
    # X <- model.matrix(mt, mf, contrasts = obj$contrasts)
    # y <- model.frame(obj)[,1]
    # X <- model.frame(obj)[,2:dim(model.frame(obj))[2]]
    olscf <- summary(lm(model.frame(obj)))$coefficients
    rownames(olscf) <- rownames(coef(obj))
    olscf <- cbind(olscf[parm, 1, drop = FALSE], olscf[parm, 
                                                       1, drop = FALSE] - olscf[parm, 2, drop = FALSE] * 
                     zalpha, olscf[parm, 1, drop = FALSE] + olscf[parm, 
                                                                  2, drop = FALSE] * zalpha)
    colnames(olscf) <- c("coefficients", "lower bd", "upper bd")
  }
  mfrow_orig <- par("mfrow")
  mar_orig <- par("mar")
  if (is.null(mfrow)) 
    mfrow <- n2mfrow(length(parm))
  if (is.null(mar)) 
    mar <- c(3.1, 3.1, 3.1, 1.6) # Bottom, left, top, right
  par(mfrow = mfrow, mar = c(3.1, 3.1, 1.1, 1.6))
  col <- rep(col, length.out = 2)
  lty <- rep(lty, length.out = 2)
  if (is.null(border)) 
    border <- col[2]
  if (is.null(main)) 
    main <- parm
  main <- rep(main, length.out = length(parm))
  xlab <- rep(xlab, length.out = length(parm))
  ylab <- rep(ylab, length.out = length(parm))
  ylim0 <- ylim
  for (i in seq(along = parm)) {
    b <- t(sapply(seq(along = cf), function(tau) cf[[tau]][i, 
                                                           ]))
    if (is.null(ylim)) {
      if (ols) 
        ylim <- range(c(b[, 2], b[, 3], olscf[i]))
      else ylim <- range(b[, 2], b[, 3])
    }
    plot(rep(taus, 2), c(b[, 2], b[, 3]), type = "n", ylim = ylim, 
         xlab = xlab[i], ylab = ylab[i], main = "", xaxt=xaxt, yaxt=yaxt, bty=bty)
    axis(1, at=taus, label=format(taus, nsmall=2))
    yaxis <- pretty(seq(ylim[1], ylim[2], length=5))
    axis(2, at=yaxis, label=format(yaxis, nsmall=2))
    
    polygon(c(taus, rev(taus)), c(b[, 2], rev(b[, 3])), col = col[2], 
            border = border)
    points(taus, b[, 1], cex = cex, pch = pch, type = type, 
           col = lineColor, lwd=1.5, ...)
    if (ols) {
      abline(h = olscf[i, 1], col = olsColor, lty = lty[1], lwd=1.5)
      abline(h = olscf[i, 2], col = olsColor, lty = lty[2], lwd=1.5)
      abline(h = olscf[i, 3], col = olsColor, lty = lty[2], lwd=1.5)
    }
    abline(h = 0, col = zeroLineColor, lwd=1.5)
    ylim <- ylim0
  }
  par(mfrow = mfrow_orig, mar = mar_orig)
  if (ols) 
    x <- c(cf, list(ols = olscf))
  else x <- cf
  invisible(structure(as.vector(unlist(x)), .Dim = c(dim(x[[1]]), 
                                                     length(x)), .Dimnames = list(rownames(x[[1]]), colnames(x[[1]]), 
                                                                                  names(x))))
}



### 2017-05-15
# Plot the iterations from the genetic algorithm (GA)
plotQRGAiterations <- function(iterationLogAddress=paste(getwd(), "/modeloutput/iteration_log/",
                               "QRGA_log__modelsize=08_horizons=12_popsize=0230_generations=00200_variancePenalization=00050_parallel=T.csv", sep=""),
                               generations=200, horizons=12) {
 iteration_df <- read.table(file=iterationLogAddress, header=TRUE, sep=",",
                            stringsAsFactors=F) 
 generation_fitnesses <- data.frame(matrix(0, generations, horizons+1))
 colnames(generation_fitnesses) <- c("Generation", paste("Horizon ", seq(1, horizons, by=1), sep=""))
 
 # property <- "MeanR1" # Can also be BestFitness, etc.
 # property_yLabel <- bquote("Avg." ~ R^1~(tau))
 property <- "BestFitness"
 property_yLabel <- "Fitness"
 
 generation_fitnesses[,1] <- seq(1, generations, by=1)
 for (i in 1:dim(iteration_df)[1]) {
   h <- iteration_df[i,1]
   gen <- iteration_df[i,2]
   generation_fitnesses[gen,1+h] <- iteration_df[property][i,]
 }
 
 # Rebase the dataset due to convergence
 generation_fitnesses <- generation_fitnesses[1:100,]
 
 # Then plot the stuff
 xlim <- c(1, dim(generation_fitnesses)[1])
 ylim <- c(min(generation_fitnesses[,2:dim(generation_fitnesses)[2]]), max(generation_fitnesses[,2:dim(generation_fitnesses)[2]]))
 xlabels <- pretty(seq(xlim[1], xlim[2], length=10), n=8)
 ylabels <- pretty(seq(ylim[1], ylim[2], length=10), n=10)
 
 pchs <- seq(0, horizons-1, by=1)
 ltys <- rep(seq(1,6, by=1), floor(horizons/6))[1:horizons]
 
 # Font and symbol sizes
 cexPch <- 1.2
 cexTitle <- 1.2
 
 def.par <- par(no.readonly = TRUE)

 pdf(paste(getwd(), '/plots/iterations/iteration-plot-', property, '.pdf', sep=""), height=6,
     width=11, family = "Palatino")
 
 par(oma = c(0, 0.5, 0, 0), mar=c(5.1, 4.1, 2.1, 1.1)) # Bottom, left, top, right
 
 layout(matrix(c(1,2), nrow = 1), widths = c(0.84, 0.16))
 
 for (i in 2:dim(generation_fitnesses)[2]) {
   if (i==2) {
     plot(generation_fitnesses[,i] ~ generation_fitnesses[,1], type="o", main="",
          xlim=xlim, ylim=ylim, xlab="Generation", ylab=as.expression(property_yLabel), bty="n", xaxt="n", yaxt="n",
          pch=c(pchs[i-1], rep(NA, 3)), cex=cexPch, lty=ltys[i-1])
     axis(1, at=xlabels, labels=format(xlabels, nsmall=0))
     axis(2, at=ylabels, labels=format(ylabels, nsmall=1))
   } else if (generation_fitnesses[1,i] > 0) {
     par(new=T)
     plot(generation_fitnesses[,i] ~ generation_fitnesses[,1], type="o", main="", xlim=xlim, ylim=ylim, axes=F,
          xlab="", ylab="", pch=c(pchs[i-1], rep(NA,3)), cex=cexPch, lty=ltys[i-1], bty="n")
   }
 }
 
 par(mar=c(0,0,0,0))
 plot(1:3, rnorm(3), pch=1, lty=1, type="n", axes=F, ann=F)
 legend("left", colnames(generation_fitnesses)[2:(horizons+1)], pch=pchs, lty=ltys)
 
 # Drop title, as the caption in LaTeX is sufficient
 #mtext(bquote(bold("Genetic algorithm (GA) best fitness per generation")),
 #        outer = TRUE, cex = cexTitle)
 
 dev.off()
 par(def.par)
 
}







































































































