# Filename:        quantile_regression_model.R
# Date:            2017-03-09
# Author:          Kristian Sandaker (kristian.sandaker@outlook.com)
# Purpose:         Functions for e.g. loading files
# Data Written:    
# Packages Used:   quantreg
# Source Files:    helpers.R

library(leaps)
library(quantreg)
source('helpers.R')
# source('latex_output.R')

bestInSampleMeanHorizonModels <- function(df_fullLagged, startDate_inSample,
                                  endDate_inSample, forecastHorizon=12,
                                  maxLmSize=10, 
                                  bss_selectionCriterion="rsq") {
  df_inSample <- df_fullLagged[df_fullLagged[,1] >= startDate_inSample &
                                 df_fullLagged[,1] <= endDate_inSample,]
  
  dates_inSample <- df_inSample[,1]
  Y_mat <- df_inSample[,2:(1+forecastHorizon)]
  X_mat <- df_inSample[,(2+forecastHorizon):dim(df_inSample)[2]]
  
  best_lms <- list()
  
  for (i in 1:dim(Y_mat)[2]) { # Loop over every n-step ahead forecast horizon
    # X_forecast <- data.frame(Y_mat[i], X_mat[,1:10])
    print(paste("Running forward selection on ", colnames(Y_mat)[i]))
    res_bss <- regsubsets(y=Y_mat[,i], x=X_mat, nbest=1, nvmax=maxLmSize,
                                 intercept = TRUE, method="forward")
    criterionList <- summary(res_bss)[[bss_selectionCriterion]]
    bestModelIndex <- which(criterionList == max(criterionList))
    bestModelSeriesNames <- data.frame(summary(res_bss)$which[bestModelIndex,])
    reg_mat <- Y_mat[i]
    for (j in 2:dim(bestModelSeriesNames)[1]) {
      rowName <- rownames(bestModelSeriesNames)[j]
      seriesBool <- bestModelSeriesNames[j,1]
      if (seriesBool) reg_mat[rowName] <- X_mat[rowName]
    }
    best_lms[[i]] <- lm(reg_mat)
  }
  return(best_lms)
}

predictMeanHorizonOutOfSample <- function(df_fullLagged, best_lms,
                              startDate_outOfSample=as.Date("2016-02-29"),
                              endDate_outOfSample=as.Date("2017-01-31"), forecastHorizon=12) {
  df_outOfSample <- df_fullLagged[df_fullLagged[,1] >= startDate_outOfSample & df_fullLagged[,1] <= endDate_outOfSample,]
  df_forecast <- df_outOfSample[,1:(forecastHorizon+1)]
  
  for (i in 1:forecastHorizon) {
    colname_forecast <- paste(strsplit(colnames(df_outOfSample)[2], "_fc")[[1]][1], "_pred.", sprintf("%02d", i-1), sep="")
    df_forecast[colname_forecast] <- t(t(rep(0, dim(df_outOfSample)[1])))
    best_lm <- best_lms[[i]]
    for (j in 1:dim(df_outOfSample)[1]) {
      df_forecast[colname_forecast][j,1] <- predict(best_lm, df_outOfSample[j,])
    }
  }
  return(df_forecast)
}

constructHorizonAllowableSeriesDf <- function(df_full,
                                              df_lagStructure,
                                              maxHorizon) {
  df_allowableSeries <- data.frame(matrix(0, maxHorizon, dim(df_full)[2]-maxHorizon-1),
                                   stringsAsFactors=F)
  colnames(df_allowableSeries) <- colnames(df_full)[(2+maxHorizon):length(colnames(df_full))]
  for (n in 1:dim(df_allowableSeries)[2]) {
    lag <- as.numeric(strsplit(strsplit(colnames(df_allowableSeries)[n], split="_lag.")[[1]][2],
                    split="_delta.")[[1]][1])
    seriesName <- strsplit(colnames(df_allowableSeries)[n], split="_lag.")[[1]][1]
    initialLags <- df_lagStructure[which(df_lagStructure[,1] == seriesName), 2:dim(df_lagStructure)[2]]
    for (k in 1:maxHorizon) {
      allowableLags     <- initialLags - k + 1
      # print(paste(seriesName, " @ lag ", lag, " @ Horizon ", k, 
      #             " has allowable lags: ", paste(allowableLags, collapse=", "), sep=""))
      if (lag %in% allowableLags) {
        df_allowableSeries[k,n] <- 1
      }
    }
  }
  return(df_allowableSeries)
}



qrRegGeneticAlgorithm <- function(df_full, df_lagStructure,
                                  df_groupStructure, 
                                  df_groupStructureConstraints,
                                  K, allowDuplicates=F,
                                  saveResult=T, 
                                  Tau = c(0.05, 0.10, 0.20, 0.50, 0.80, 0.90, 0.95),
                                  parallel=T, nCores=0, verbose=F,
                                  parallelLogFile=T,
                                  variance_penalization=50, population_size=115,
                                  generations=100,
                                  writeGenerationLog=T, attempts=1) {
  # Parametrization
  elitism <- 0.10
  max_model_size <- 8
  K <- 12 # Number of horizons
  
  # Allowable series per horizon
  df_horizonAllowableSeries <- constructHorizonAllowableSeriesDf(df_full, df_lagStructure, K)
  
  # Constraint violation parameters
  model_size_penalization       <- 5 ## Penalization of the model size
  illegal_lag_penalization      <- 5 ## Penalization of illegal lags
  group_structure_penalization  <- 5 ## Penalization of group structure violations
  duplicate_penalization        <- 5 ## Penalization of duplicate series (at e.g. different lags)
  
  # Tournament params
  tournament_size <- 2
  tournament_prob <- 0.80
  crossover_prob <- 0.60
  mutation_prob <- 0.80
  mutation_add_prob <- 0.80
  
  # Parallelization
  parallelString <- if (parallel) "T" else "F"
  
  # Write iterations to file
  if (writeGenerationLog) {
    generationLog_df <- data.frame(matrix(0, K*generations*attempts, 5+3*length(Tau)))
    colnames(generationLog_df) <- c("Attempt", "Horizon", "Generation", "TimeElapsed",
                                    "BestFitness", paste("R1Tau=", Tau, sep=""),
                                    paste("AICTau=", Tau, sep=""), paste("BICTau=", Tau, sep=""))
    generationLog_counter <- 1
    # k, generation, time elapsed, min-fitness, 
  }
  
  if (parallel) {
    require(parallel)
    if (nCores == 0) {
      nCores <- detectCores() - 1
    } 
    if (parallelLogFile) {
      logFileAddress <- paste(getwd(), "/modeloutput/log/log.txt", sep="")
      print(paste("Setting up log file: ", logFileAddress, sep=""))
      if (file.exists(logFileAddress)) file.remove(logFileAddress) # Delete current log.
      cl <- makeCluster(nCores, methods=F, outfile=logFileAddress)
    } else {
      cl <- makeCluster(nCores, methods=F)
    }
    
    # cl <- makeCluster(nCores, methods=F, outfile)
    clusterExport(cl, varlist = c("population", "fitnesses", "evaluateQrPhiTauLoss", 
                                  "K", "Tau", "df_full", "variance_penalization",
                                  "max_model_size", "model_size_penalization",
                                  "illegal_lag_penalization", "group_structure_penalization",
                                  "allowDuplicates", "duplicate_penalization",
                                  "df_horizonAllowableSeries", "df_groupStructureConstraints",
                                  "df_groupStructure", "tournament", "tournament_size",
                                  "tournament_prob", "crossover_procedure", "crossover",
                                  "crossover_prob", "mutate", "mutate_feasible", "mutation_prob",
                                  "mutation_add_prob"))
  }
  
  # k is the particular horizon
  # K is all horizons
  evaluateQrPhiTauLoss <- function(phi, k)
  {
    phi <- which(phi==1) + 1 + K
    require(quantreg)
    qrLossVector <- c()
    # First the Quantile Regression Loss
    for (i in 1:length(Tau)) {
      tau <- Tau[i]
      quantreg_res <- rq(data.frame(df_full[,1 + k], df_full[phi]), tau=tau)
      beta_0 <- quantreg_res$coefficients[1]
      beta <- t(t(quantreg_res$coefficients[2:length(quantreg_res$coefficients)]))
      for (t in 1:dim(df_full)[1]) {
        Y_t <- df_full[t,1 + k]
        rho <- tau - (Y_t < (beta_0 + as.matrix(df_full[phi][t,]) %*% beta))
        qrLossVector <- c(qrLossVector, rho*(Y_t - (beta_0 + as.matrix(df_full[phi][t,]) %*% beta)))
      }
    }
    # print(paste("Sum: ", sum(qrLossVector)))
    # print(paste("Variance: ", variance_penalization*var(qrLossVector)))
    
    loss <- sum(qrLossVector) + variance_penalization*var(qrLossVector)
    
    ## Then add constraint violations to the loss function
    # Model size penalty
    loss <- loss + max(length(phi) - max_model_size, 0)*model_size_penalization
    # Lag violations
    for (i in 1:length(colnames(df_full[phi]))) {
      colname_model <- colnames(df_full[phi])[i]
      # print(paste("Checking ", colname_model, sep=""))
      if (df_horizonAllowableSeries[colname_model][k,1] == 0) {
        # print(paste(colname_model, " IS violated @ lag ", k, sep=""))
        loss <- loss + illegal_lag_penalization
      }
      # colname_right <- paste(strsplit(colname_model, "_lag.")[[1]][1], "_lag.",
      #                    sprintf("%02d", as.numeric(strsplit(strsplit(colname_model, "_lag.")[[1]][2], "_delta.")[[1]][1]) + (k - 1)),
      #                    "_delta.", strsplit(colname_model, "_delta.")[[1]][2], sep="")
      # if (!(colname_right %in% colnames(df_full))) {
      #   loss <- loss + illegal_lag_penalization
      # print(paste("Colname model: ", colname_model, sep=""))
      # print(paste("Colname right: ", colname_right, sep=""))
      # print("Penalizing illegal lag.")
      # }
    }
    # Group structure violations
    groupStructureCounter <- data.frame(matrix(0, dim(df_groupStructureConstraints)[1],
                                               dim(df_groupStructureConstraints)[2]))
    groupStructureCounter[,1] <- df_groupStructureConstraints[,1]
    groupStructureCounter[,3] <- df_groupStructureConstraints[,3]
    # Increment the counter
    for (i in 1:length(colnames(df_full[phi]))) {
      seriesName <- strsplit(colnames(df_full[phi])[i], split="_lag.")[[1]][1]
      groupStructureCounter[df_groupStructure[which(df_groupStructure[,1] == seriesName), 2], 2] <- groupStructureCounter[df_groupStructure[which(df_groupStructure[,1] == seriesName), 2], 2] + 1
    }
    # Then penalize
    for (i in 1:dim(groupStructureCounter)[1]) {
      loss <- loss + max(0, groupStructureCounter[i,2] - groupStructureCounter[i,3])*group_structure_penalization
      # print(paste("Penalizing group structure ", i , ": ", max(0, groupStructureCounter[i,2] - groupStructureCounter[i,3])*group_structure_penalization), sep="")
    }
    ## Duplicates
    if (!allowDuplicates) {
      seriesNames <- c()
      for (i in 1:length(colnames(df_full[phi]))) {
        seriesNames <- c(seriesNames, strsplit(colnames(df_full[phi])[i], split="_lag.")[[1]][1])
      }
      loss <- loss + sum(duplicated(seriesNames))*duplicate_penalization
      # print(paste("Penalizing duplicates: ", sum(duplicated(seriesNames))*duplicate_penalization))
    }
    return(loss)
  }
  
  # K-ary tournament
  tournament <- function(selection_pop, fitnesses, subPopulation_size = -1) {
    # print(paste("class(selection_pop)=", class(selection_pop), "    Values=", paste(selection_pop, collapse=", "), sep=""))
    if (subPopulation_size < 1) {
      tournament_pop <- rep(0, population_size - round(elitism*population_size))  
    } else {
      tournament_pop <- rep(0, subPopulation_size)  
    }
    for (i in 1:length(tournament_pop)) {
      tourn <- sort(sample(selection_pop, size = tournament_size, replace = T))  # Change 12.04.2017 - 18:58, due to "cannot take a sample larger than..." error
      if (runif(n=1) < tournament_prob) {
        tournament_pop[i] <- tourn[which(fitnesses[tourn]==min(fitnesses[tourn]))][1]
      } else {
        tournament_pop[i] <- sample(tourn, size=1)
      }
    }
    return(tournament_pop)
  }
  
  # Simple crossover
  crossover <- function(parent1, parent2) {
    indices1 <- which(parent1==1)
    indices2 <- which(parent2==1)
    child_indices <- sample(unique(c(indices1, indices2)), size=min(length(unique(c(indices1, indices2))), max_model_size))
    child <- rep(0, length(parent1))
    child[child_indices] <- 1
    return(child)
  }
  
  # Simple mutation
  mutate <- function(child) {
    if (runif(n=1) < mutation_prob) {
      oldIndex <- sample(which(child==1), size=1)
      newIndex <- sample(1:length(child), size=1)
      child[oldIndex] <- 0
      child[newIndex] <- 1
    }
    return(child)
  }
  
  # Simple mutation, only allowable series
  mutate_feasible <- function(child, k) {
    # To avoid empty models...
    if (sum(child) <= 2) {
      allowableIndices <- which(df_horizonAllowableSeries[k,] == 1)
      newIndices <- sample(allowableIndices, size=2)
      child[newIndices] <- 1 
    }
    if (runif(n=1) < mutation_prob) {
      allowableIndices <- which(df_horizonAllowableSeries[k,] == 1)
      if (sum(which(child==1)) > 0) { # Change 12.04.2017 - 18:58, due to "cannot take a sample larger than..." error
        oldIndex <- sample(which(child==1), size=1)
        child[oldIndex] <- 0
      }
      newIndex <- sample(allowableIndices, size=1)
      child[newIndex] <- 1
    }
    # Also add variables with some small prob.
    if (((sum(child) < max_model_size) && (runif(n=1) < mutation_add_prob)) || (sum(child) <= 4)) {
      if (verbose) {
        print(paste(Sys.info()[['nodename']], '-', Sys.getpid(), ': Adding extra variables to child with ', sum(child), ' variables...', sep=''))
      }
      # print("ADDING SHIET")
      allowableIndices <- which(df_horizonAllowableSeries[k,] == 1)
      newIndex <- sample(allowableIndices, size=1)
      child[newIndex] <- 1
    }
    return(child)
  }
  
  # Full crossover procedure
  crossover_procedure <- function(pre_crossover_population, k) {
    n_children <- dim(pre_crossover_population)[1]
    post_crossover_population <- matrix(0, n_children, dim(pre_crossover_population)[2])
    for(i in 1:n_children) {
      parent1 <- pre_crossover_population[sample(1:n_children, size=1),]
      parent2 <- pre_crossover_population[sample(1:n_children, size=1),]
      # First crossover
      if (runif(n=1) < crossover_prob) {
        child <- crossover(parent1, parent2)
      } else {
        child <- if (runif(n=1) < 0.5) parent1 else parent2
      }
      # Then mutation
      child <- mutate_feasible(child, k)
      # child <- mutate(child)
      
      if (sum(child) <= 4) {
        
        allowableIndices <- which(df_horizonAllowableSeries[k,] == 1)
        newIndices <- sample(allowableIndices, size=2)
        child[newIndices] <- 1 
      }
      
      post_crossover_population[i,] <- child
    }
    return(post_crossover_population)
  }
  
  Phi_mat <- data.frame(matrix(0, K, dim(df_full)[2] - 1 - K))
  colnames(Phi_mat) <- colnames(df_full)[(K+2):dim(df_full)[2]]
  
  # Genetic algorithm
  k_populations <- list()
  t0 <- proc.time()
  
  # Code for parallelization
  parallelGAIteration <- function(tournament_pop_size, k) {
    if (verbose) {
      print(paste(Sys.info()[['nodename']], '-', Sys.getpid(), ': Running tournament @ k=', k, '...', sep=''))
    }
    tournament_pop <- tournament(seq(1:length(fitnesses)), fitnesses,
                                 subPopulation_size = tournament_pop_size)
    pre_crossover_population  <- population[tournament_pop,]
    if (verbose) {
      print(paste(Sys.info()[['nodename']], '-', Sys.getpid(), ': Running crossover @ k=', k, '...', sep=''))
    }
    post_crossover_population <- crossover_procedure(pre_crossover_population, k=k)
    return(post_crossover_population)
  }
  
  # Code for parallel fitness calculation
  parallelEvaluateFitness <- function(populationChunk, k) {
    return(apply(populationChunk, 1, evaluateQrPhiTauLoss, k=k))
  }
  
  for (attempt_index in 1:attempts) {
  
  for (k in 1:K) {
    print(sprintf("%18s %4d %5s", "### Running GA for horizon", k, " ###"))
    
    # Initialize
    population <- matrix(0, population_size, dim(Phi_mat)[2])
    for (i in 1:dim(population)[1]) {
      phi_vec_ind <- sample(1:dim(Phi_mat)[2], max_model_size, replace=F)
      phi_vec <- matrix(0, 1, dim(Phi_mat)[2])
      phi_vec[,phi_vec_ind] <- 1
      population[i,] <- phi_vec
    }
    print(paste("### Initialization of ", dim(population)[1], "x",
                dim(population)[2], " population done ###", sep=""))
    
    fitnesses <- NULL
    
    for (g in 1:generations) {
      # Parallization
      if (parallel) {
        # t0 <- proc.time()[3]
        
        # clusterExport(...) assigns the values on the master R process of the 
        # variables named in varlist to variables of the same names in the 
        # global environment (aka ‘workspace’) of each node
        
        if (g == 1) { # Evaluate the fitness initially for the first generation
          # Generate the individuals to be sent to each cluster for fitness calculation
          popList <- list()
          individuals_per_core <- ceiling(population_size/nCores)
          if (population_size %% individuals_per_core > 0) {
            coresUtilized <- (population_size - (population_size %% individuals_per_core))/individuals_per_core + 1
          } else {
            coresUtilized <- population_size/individuals_per_core
          }
          # if ((k >= 8) || verbose) {
          #   print(paste("There are ", individuals_per_core, " individuals per core, ",
          #               "population size ", population_size, 
          #               ", and ", coresUtilized, " cores utilized.", sep=""))
          # }
          
          for (l in 1:coresUtilized) {
            if (l==1) {
              popList[[l]] <- population[1:individuals_per_core,]
            } else if (l==coresUtilized) {
              popList[[l]] <- population[(individuals_per_core*(l-1)+1):population_size,]
            } else {
              popList[[l]] <- population[(individuals_per_core*(l-1)+1):(individuals_per_core*l),]
            }
          }
          
          # if ((k >= 8) || verbose) {
          #   print(paste("The length of popList is ", length(popList), 
          #               ", with element size ", dim(popList[[1]])[1],
          #               ", last element size ", dim(popList[[length(popList)]])[1],
          #               ". The actual population size is ", dim(population)[1], ".", sep=""))
          # }
          
          fitnesses <- parLapply(cl, popList, parallelEvaluateFitness, k=k)
          fitnesses <- unlist(fitnesses)
          clusterExport(cl, "fitnesses")
        }
        # Crossover, mutation, etc.
        elite_population <- population[order(fitnesses)[1:round(elitism*population_size)],, drop=F]
        tournament_population_size <- population_size - dim(elite_population)[1]
        
        cluster_population_size <- ceiling(tournament_population_size/nCores)
        tournament_population <- list()
        pop_counter <- 0
        for (l in 1:nCores) {
          if (pop_counter + cluster_population_size < tournament_population_size) {
            tournament_population[[l]] <- cluster_population_size
          } else {
            tournament_population[[l]] <- tournament_population_size - pop_counter
          }
          pop_counter <- pop_counter + tournament_population[[l]]
        }
        
        # Then do the multicore crossover procedure
        # if ((k >= 8) || verbose) {
        #   print("Running parallel GA iteration on the population...")
        # }
        
        post_crossover_population <- parLapply(cl, tournament_population, parallelGAIteration, k=k)
        population <- elite_population
        for (l in 1:length(post_crossover_population)) {
          if (!is.null(dim(post_crossover_population[[l]]))) {
            if (dim(population)[1] + dim(post_crossover_population[[l]])[1] < population_size) {
              population <- rbind(population, post_crossover_population[[l]])
            } else if (dim(population)[1] < population_size) {
              population <- rbind(population, post_crossover_population[[l]][1:min((population_size-dim(population)[1]),
                                                                                   dim(post_crossover_population[[l]])[1]),])
            }
          }
        }
        
        # Generate the individuals to be sent to each cluster for crossover and mutation
        popList <- list()
        individuals_per_core <- ceiling(population_size/nCores)
        if (population_size %% individuals_per_core > 0) {
          coresUtilized <- (population_size - (population_size %% individuals_per_core))/individuals_per_core + 1
        } else {
          coresUtilized <- population_size/individuals_per_core
        }
        # if ((k >= 8) || verbose) {
        #   print(paste("There are ", individuals_per_core, " individuals per core, ",
        #               "population size ", population_size, 
        #               ", and ", coresUtilized, " cores utilized.", sep=""))
        # }
        
        for (l in 1:coresUtilized) {
          if (l==1) {
            popList[[l]] <- population[1:individuals_per_core,]
          } else if (l==coresUtilized) {
            popList[[l]] <- population[(individuals_per_core*(l-1)+1):population_size,]
          } else {
            popList[[l]] <- population[(individuals_per_core*(l-1)+1):(individuals_per_core*l),]
          }
        }
        
        # if ((k >= 8) || verbose) {
        #   print(paste("The length of popList is ", length(popList), 
        #               ", with element size ", dim(popList[[1]])[1],
        #               ", last element size ", dim(popList[[length(popList)]])[1],
        #               ". The actual population size is ", dim(population)[1], ".", sep=""))
        # }
        # Calculate the fitnesses
        fitnesses <- parLapply(cl, popList, parallelEvaluateFitness, k=k)
        fitnesses <- unlist(fitnesses)
        
        # Export new information to cluster
        clusterExport(cl, varlist = c("population", "fitnesses"))
        
      } else { # No parallelization´
        if (g==1) {
          fitnesses <- apply(population, 1, evaluateQrPhiTauLoss, k=k)
        }
        elite_population <- population[order(fitnesses)[1:round(elitism*population_size)],, drop=F]
        elite_fitnesses <- apply(elite_population, 1, evaluateQrPhiTauLoss, k=k)
        tournament_pop <- tournament(seq(1:population_size), fitnesses)
        pre_crossover_population <- population[tournament_pop,]
        post_crossover_population <- crossover_procedure(pre_crossover_population, k=k)
        population <- rbind(elite_population, post_crossover_population)
        elite_fitnesses <- apply(elite_population, 1, evaluateQrPhiTauLoss, k=k)
        fitnesses <- apply(population, 1, evaluateQrPhiTauLoss, k=k)
      }
      
      ### Generation output
      totalCores <- if (parallel) nCores else 1
      if (!parallel) {
        coresUtilized <- 1
      }
      
      
      ## Find the R1 range
      
      r1_vec <- rep(0, length(Tau))
      aic_vec <- rep(0, length(Tau))
      bic_vec <- rep(0, length(Tau))
      # aic_vec <- rep(0, length(Tau))
      phi <- population[which(fitnesses == min(fitnesses))[1],]
      phi <- which(phi==1) + 1 + K
      require(quantreg)
      for (i in 1:length(Tau)) {
        best_quantreg_tau <- Tau[i]
        best_quantreg_res <- rq(data.frame(df_full[,1 + k], df_full[phi]), tau=best_quantreg_tau)
        r1_vec[i] <- pseudo_rsq(best_quantreg_res)
        aic_vec[i] <- AIC(best_quantreg_res, k=2)
        bic_vec[i] <- AIC(best_quantreg_res, k=log(length(best_quantreg_res$fitted.values)))
        # aic_vec[i] <- aic(aic, AIC(qrModels[[i]][[j]], k=2))
      }
      r1_vec_string <- paste(format(round(min(r1_vec), 2), nsmall=2), "-",
                             format(round(max(r1_vec), 2), nsmall=2), " @ SD=", format(round(sd(r1_vec), 4), nsmall=4), sep="")
      
      t_elapsed <- as.numeric((proc.time() - t0)[3])
      
      ## Nice printouts
      print(sprintf("%10s %6d %10s %6d %4s %6s %16s %4.2f %12s %13s %16s %10.2fs %10s %8s",
                    "Steps ahead:", k, "Generation:", g, 
                    "|P|:", paste(dim(population)[1], "/", population_size, sep=""),
                    "Best fitness:", min(fitnesses), # "Worst fitness:", max(fitnesses),
                    "R1 range:", r1_vec_string,
                    "Time elapsed:", t_elapsed,
                    "Cores utilized:", paste(coresUtilized, "/", totalCores, sep="")))
      
      ## Write iteration log
      if (writeGenerationLog) {
        logDataAddress <- paste(getwd(), "/modeloutput/iteration_log/QRGA_fullR1log_",
                                "_modelsize=", sprintf("%02d", max_model_size),
                                "_horizons=", sprintf("%02d", K),
                                "_popsize=", sprintf("%04d", population_size), 
                                "_generations=", sprintf("%05d", generations),
                                "_variancePenalization=", sprintf("%05d", variance_penalization),
                                "_parallel=", parallelString, ".csv", sep="")
        
        generationLog_df[generationLog_counter,1] <- attempt_index
        generationLog_df[generationLog_counter,2] <- k
        generationLog_df[generationLog_counter,3] <- g
        generationLog_df[generationLog_counter,4] <- t_elapsed
        generationLog_df[generationLog_counter,5] <- min(fitnesses)
        generationLog_df[generationLog_counter,6:(6+length(Tau)-1)] <- r1_vec
        generationLog_df[generationLog_counter,(6+length(Tau)):(6+2*length(Tau)-1)] <- aic_vec
        generationLog_df[generationLog_counter,(6+2*length(Tau)):(6+3*length(Tau)-1)] <- bic_vec
        
        write.table(generationLog_df, file=logDataAddress, row.names=F, sep=",")
        
        generationLog_counter <- generationLog_counter + 1
        
      }
      
    }
    phi <- population[order(fitnesses)[1],]
    phi <- which(phi==1) + 1 + K
    phi_cols <- df_full[phi]
    k_populations[[k]] <- phi_cols # Save the population for later on
    if (saveResult) {
      res_df <- data.frame(horizon=seq(1:K), series_name=matrix("",K,round(max_model_size*1.5)),
                           stringsAsFactors=F)
      for (i in 1:length(k_populations)) {
        phi_colnames <- colnames(k_populations[[i]])
        for (j in 1:length(phi_colnames)) {
          colname <- phi_colnames[j]
          res_df[i,(j+1)] <- colname
        }
      }
      
      write.table(res_df, file=paste(getwd(), "/modeloutput/QRGA",
                                     "_modelsize=", sprintf("%02d", max_model_size),
                                     "_horizons=", sprintf("%02d", K),
                                     "_popsize=", sprintf("%04d", population_size), 
                                     "_generations=", sprintf("%05d", generations),
                                     "_variancePenalization=", sprintf("%05d", variance_penalization),
                                     "_parallel=", parallelString,
                                     "_writeGenerationLog=", writeGenerationLog,
                                     "_attempt=", attempt_index, ".csv", 
                                     sep=""), row.names=F, sep=",")
    }
  }
    
  }
  # Stop clusters, done
  stopCluster(cl)
  return(k_populations)
}



constructQrModelsFromGaRun <- function(df_full, horizon_series,
                                       Tau = c(0.05, 0.10, 0.20, 0.50, 0.80, 0.90, 0.95)) {
  require(quantreg)
  qrModels <- list()
  for (k in 1:length(horizon_series)) {
    phi_cols <- horizon_series[[k]]
    tauModels <- list()
    for (i in 1:length(Tau)) {
      tau <- Tau[i]
      tauModels[[i]] <- rq(data.frame(df_full[,1+k], phi_cols), tau=tau)
    }
    qrModels[[k]] <- tauModels
  }
  return(qrModels)
}

constructQrModelsFromGaFile <- function(df_full, gaFileName,
                                        Tau = c(0.05, 0.10, 0.20, 0.50, 0.80, 0.90, 0.95),
                                        maxHorizons = 12) {
  res_df <- read.table(file=gaFileName, header=TRUE, sep=",", 
                       stringsAsFactors = F, na.strings=c("", "NA"))
  horizon_series <- list()
  for (i in 1:maxHorizons) {
    k <- res_df[i,1]
    for (j in 2:dim(res_df)[2]) {
      if (j==2) {
        series_names <- res_df[i,j]
      } else if (!is.na(res_df[i,j])) {
        series_names <- c(series_names, res_df[i,j])
      }
    }
    phi_cols <- df_full[series_names]
    horizon_series[[k]] <- phi_cols
  }
  return(constructQrModelsFromGaRun(df_full, horizon_series, Tau=Tau))
}

pseudo_rsq <- function(rq_model) {
  restr_model <- rq(rq_model$y ~ 1, tau=rq_model$tau)
  # rho <- function(u, tau=rq_model$tau)u*(tau - (u < 0))
  R1 <- 1 - rq_model$rho/restr_model$rho
  return(R1)
}

qreset <- function(rq_model, order, R=1000) {
  if (order < 2) {
    print("Order must be equal or larger than 2")
    return()
  }
  rq_model_df <- data.frame(cbind(rq_model$y, rq_model$x[,2:dim(rq_model$x)[2]]))
  for (i in 2:order) {
    rq_model_df <- data.frame(rq_model_df, cbind(rq_model$y^i))
  }
  aux_rq_model <- rq(rq_model_df, tau=rq_model$tau)
  # A list consisting of two elements: A matrix B of dimension R by p is 
  # returned with the R resampled estimates of the vector of quantile regression
  # parameters. A matrix U of sampled indices (for bsmethod in 
  # c("xy", "wxy")) or gradient evaluations.
  # rq_bootstrap <- boot.rq(x=aux_rq_model$x, y=aux_rq_model$y, tau=rq_model$tau, R=200, bsmethod="pwy")
  n_coef <- length(aux_rq_model$coefficients)
  if (order == 2) {
    return(summary(aux_rq_model, se="boot", tau=rq_model$tau, R=R, bsmethod="pwy")$coefficients[n_coef,3])
  } else {
    alpha_vec <- cbind(aux_rq_model$coefficients[(n_coef-order+2):n_coef])
    coef_boot <- boot.rq(x=aux_rq_model$x, y=aux_rq_model$y, se="boot", tau=rq_model$tau, R=R, bsmethod="pwy")[[1]]
    V_mat <- cov(coef_boot[,(n_coef-order+2):n_coef])
    wald_statistic <- t(alpha_vec) %*% solve(V_mat) %*% alpha_vec
    return(wald_statistic)
  }
}

qreset_to_string <- function(rq_model, order, R=1000,
                             chi_sq_significance_levels=c(0.10, 0.05, 0.01),
                             n_decimals=2) {
  if (order < 2) {
    print("Need QRESET(2) or higher...")
    return("")
  }
  statistic <- qreset(rq_model, order, R)
  chiSqStars <- ""
  for (i in 1:length(chi_sq_significance_levels)) {
    sign_level <- 1-chi_sq_significance_levels[i]
    if (order > 2) {
      if (qchisq(sign_level, df=(order-1)) < statistic) {
        chiSqStars <- paste("^{", paste(rep("*", i), collapse=""), "}", sep="")
      }
    } else if (order == 2) { # Use t-statistic
      t_df <- length(rq_model$fitted.values) - length(rq_model$coefficients) - 1
      if (qt(sign_level, df=t_df) < statistic) {
        chiSqStars <- paste("^{", paste(rep("*", i), collapse=""), "}", sep="")
      }
    }
  }
  return(paste("$", format(round(statistic, n_decimals), nsmall=n_decimals), chiSqStars, "$", sep=""))
}


# qrModels1 <- constructQrModelsFromGaRun(df_full, k_populations)
# qrNamesTable <- constructQrNamesTable(qrModels2)
# qrMetricsTable <- constructQrMetricsTable(qrModels2)


 
# dataDir <- '/Users/Kristian/Kristian/Dropbox/Studier/9. semester/Prosjektoppgave/Prosjektoppgave shared/Master Thesis/Model/Data'
# dataBaseName <- 'Database_Master_Monthly_2017-03-07_01a.csv'
# 
# dataFrameAddress <- paste(dataDir, '/', dataBaseName, sep="")
# 
# minDate <- as.Date('2005-01-01')
# maxDate <- as.Date('2017-01-31')
# 
# df <- importDataFrame(dataFrameAddress=dataFrameAddress,
#                       startDate=minDate, endDate=maxDate, firstColDateCol=T,
#                       dateFormat='%d/%m/%Y',
#                       excludeVector=c('norwegianexports_norgessjomatrad', 
#                                       'forward.eur.Xm_fishpool'),
#                       convertToReturns='log')
# plot(df[,2] ~ df[,1], type='l', main='NASDAQ')
# 
# nqsalmon <- df[,2]
# numind <- df[,3]
# biomass <- df[,4]
# meatindex <- df[,14]
# harvestvolume <- df[,7]
# 
# 
# T <- dim(df)[1]
# biomass <- df[1:(T-0),7]
# nqsalmon <- df[1:T, 2]
# 
# 
# 
# taus <- c(.05,.1,.25,.75,.9,.95)
# func <- rq((nqsalmon) ~ (biomass), tau=taus)
# 
# 
# 
# xx <- seq(min(biomass),max(biomass),0.01)
# f <- coef(func)
# yy <- cbind(1,xx) %*% f
# 
# plot(nqsalmon ~ biomass)
# 
# cols <- c("green", "red", "blue", "blue", "red", "green")
# ltys <- c(1,2,3,3,2,1)
# 
# for (i in 1:length(taus)) {
#   tau <-taus[i]
#   lines(xx, yy[,i], lty=ltys[i], col=cols[i])
# }
# meanline <- lm(df[,2] ~ df[,26] + df[,7])
# summary(meanline)
# 
# abline(meanline,col="red",lty = 2, lwd=2)
# abline(rq(nqsalmon ~ biomass), col="blue", lwd=2)
# 
# 
# 
# 
# colnames(df)